BEST_MODEL_FILE = 'best_model.photonai'
BEST_CONFIG_PREDICTIONS_FILE = 'best_config_predictions.csv'
RESULTS_FILE = 'photonai_results.json'
SUMMARY_FILE = 'photonai_summary.txt'
BLUEPRINT_FILE = '_optimum_pipe_blueprint.pkl'
META_FILE = '_optimum_pipe_meta.pkl'
HYPERPIPE_CONFIG_FILE = 'hyperpipe_config.json'